import { STORAGE_PREFIX } from '../constants';
import { InvalidStateError } from '../errors';
var Authorization = /** @class */ (function () {
    /**
     * Represents the Authorization response issued by the authentication server as a `code` and a
     * `state`. The `state` matches the one from the AuthorizationRequest and is used to load the
     * `code_verifier` that the request has stored in client's storage.
     * @param options
     */
    function Authorization(options) {
        this.code = options.code;
        this.state = options.state;
    }
    Object.defineProperty(Authorization.prototype, "isLoaded", {
        get: function () {
            return !!this.code && !!this.scope;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Returns an AuthorizationSignature from this Authorization. Loads the code_verifier
     * from client's storage if not loaded already.
     * @throws {InvalidStateError} if no matching state found in storage.
     */
    Authorization.prototype.getSignature = function () {
        if (!this.isLoaded) {
            this.loadVerifier();
        }
        return {
            code: this.code,
            code_verifier: this.code_verifier,
        };
    };
    /**
     * Load the code verifier stored for this authorization's state in client's storage and removes
     * it from storage.
     * @throws {InvalidStateError} if no matching state found in storage.
     */
    Authorization.prototype.loadVerifier = function () {
        var stored = window.localStorage.getItem(STORAGE_PREFIX + this.state);
        // nothing stored for given state
        if (!stored) {
            throw new InvalidStateError("No code verifier for this state (" + this.state + ").");
        }
        // remove from localStorage
        window.localStorage.removeItem(STORAGE_PREFIX + this.state);
        var storedJSON = JSON.parse(stored);
        this.code_verifier = storedJSON.v;
        this.scope = storedJSON.s;
    };
    return Authorization;
}());
export { Authorization };
//# sourceMappingURL=Authorization.js.map