var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { encodeState, bufferToBase64UrlEncoded } from '../utils/base64';
import { createRandomString, sha256 } from '../utils/crypto';
import { createQueryParams } from '../utils/queryString';
import { STORAGE_PREFIX } from '../constants';
var AuthorizationRequest = /** @class */ (function () {
    /**
     * Represent an authorization request that is sent to the authentication server. The request
     * is composed of static values given in the `options` parameter and dynamic, random values for
     * `state` and `code_verifier` which are set upon creation.
     * @param options
     */
    function AuthorizationRequest(options) {
        this.code_challenge_method = 'S256';
        this.response_type = 'code';
        this.client_id = options.client_id;
        this.redirect_uri = options.redirect_uri;
        this.scope = options.scope;
        this.state = encodeState(createRandomString());
        this.code_verifier = createRandomString();
    }
    /**
     * Returns the request parameters as an object.
     */
    AuthorizationRequest.prototype.parameters = function () {
        return __awaiter(this, void 0, void 0, function () {
            var codeChallenge;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getCodeChallenge()];
                    case 1:
                        codeChallenge = _a.sent();
                        return [2 /*return*/, {
                                client_id: this.client_id,
                                redirect_uri: this.redirect_uri,
                                response_type: this.response_type,
                                scope: this.scope,
                                state: this.state,
                                code_challenge: codeChallenge,
                                code_challenge_method: this.code_challenge_method,
                            }];
                }
            });
        });
    };
    /**
     * Returns the request parameters as a query string.
     */
    AuthorizationRequest.prototype.asQueryString = function () {
        return __awaiter(this, void 0, void 0, function () {
            var params;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.parameters()];
                    case 1:
                        params = _a.sent();
                        return [2 /*return*/, createQueryParams(params)];
                }
            });
        });
    };
    /**
     * Calculate the code challenge base on `code_verifier`'s value or get it from cache.
     */
    AuthorizationRequest.prototype.getCodeChallenge = function () {
        return __awaiter(this, void 0, void 0, function () {
            var code_challengeBuffer;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!!this.code_challenge) return [3 /*break*/, 2];
                        return [4 /*yield*/, sha256(this.code_verifier)];
                    case 1:
                        code_challengeBuffer = _a.sent();
                        this.code_challenge = bufferToBase64UrlEncoded(code_challengeBuffer);
                        _a.label = 2;
                    case 2: return [2 /*return*/, this.code_challenge];
                }
            });
        });
    };
    /**
     * Store the code verifier and requested scope in the client's storage.
     */
    AuthorizationRequest.prototype.storeState = function () {
        window.localStorage.setItem(STORAGE_PREFIX + this.state, JSON.stringify({ v: this.code_verifier, s: this.scope }));
    };
    /**
     * Remove the code verifier and requested scope from the client's storage.
     */
    AuthorizationRequest.prototype.clearState = function () {
        window.localStorage.removeItem(STORAGE_PREFIX + this.state);
    };
    return AuthorizationRequest;
}());
export { AuthorizationRequest };
//# sourceMappingURL=AuthorizationRequest.js.map