import { AuthorizationSignature } from '../interfaces/AuthorizationSignature';
export declare class Authorization {
    code: string;
    state: string;
    code_verifier?: string;
    scope?: string;
    private get isLoaded();
    /**
     * Represents the Authorization response issued by the authentication server as a `code` and a
     * `state`. The `state` matches the one from the AuthorizationRequest and is used to load the
     * `code_verifier` that the request has stored in client's storage.
     * @param options
     */
    constructor(options: {
        code: string;
        state: string;
    });
    /**
     * Returns an AuthorizationSignature from this Authorization. Loads the code_verifier
     * from client's storage if not loaded already.
     * @throws {InvalidStateError} if no matching state found in storage.
     */
    getSignature(): AuthorizationSignature;
    /**
     * Load the code verifier stored for this authorization's state in client's storage and removes
     * it from storage.
     * @throws {InvalidStateError} if no matching state found in storage.
     */
    private loadVerifier;
}
