import { AuthorizationRequestOptions } from '../interfaces/AuthorizationRequestOptions';
export declare class AuthorizationRequest implements AuthorizationRequestOptions {
    client_id: string;
    redirect_uri: string;
    scope: string;
    state: string;
    code_verifier: string;
    readonly code_challenge_method: string;
    readonly response_type: string;
    private code_challenge?;
    /**
     * Represent an authorization request that is sent to the authentication server. The request
     * is composed of static values given in the `options` parameter and dynamic, random values for
     * `state` and `code_verifier` which are set upon creation.
     * @param options
     */
    constructor(options: AuthorizationRequestOptions);
    /**
     * Returns the request parameters as an object.
     */
    parameters(): Promise<any>;
    /**
     * Returns the request parameters as a query string.
     */
    asQueryString(): Promise<string>;
    /**
     * Calculate the code challenge base on `code_verifier`'s value or get it from cache.
     */
    getCodeChallenge(): Promise<string>;
    /**
     * Store the code verifier and requested scope in the client's storage.
     */
    storeState(): void;
    /**
     * Remove the code verifier and requested scope from the client's storage.
     */
    clearState(): void;
}
