import { DEFAULT_AUTHORIZE_TIMEOUT_IN_SECONDS } from './../constants';
/**
 * Inject an iframe to the DOM with the given url as `src` attribute. When the `'DOMContentLoaded'`
 * event is fired on the iframe, resolves on its `location.search` value and remove the iframe from
 * the DOM. The function rejects if the event is not captured before the given `timeoutInSeconds`.
 * @param url
 * @param timeoutInSeconds
 */
export var runIframe = function (url, timeoutInSeconds) {
    if (timeoutInSeconds === void 0) { timeoutInSeconds = DEFAULT_AUTHORIZE_TIMEOUT_IN_SECONDS; }
    return new Promise(function (resolve, reject) {
        // make iframe element
        var iframe = window.document.createElement('iframe');
        iframe.setAttribute('width', '0');
        iframe.setAttribute('height', '0');
        iframe.style.display = 'none';
        // timeout fallback
        var cleanupTimeout = setTimeout(function () {
            if (window.document.body.contains(iframe)) {
                window.document.body.removeChild(iframe);
            }
            reject('Timeout');
        }, timeoutInSeconds * 1000);
        // handle iframe loaded
        var iframeLoadedHandler = function () {
            if (window.document.body.contains(iframe)) {
                var search = '';
                try {
                    search = iframe.contentWindow ? iframe.contentWindow.location.search : '';
                }
                catch (_a) {
                    // iframe cannot be accessed
                }
                // remove iframe and resolve
                clearTimeout(cleanupTimeout);
                window.document.body.removeChild(iframe);
                resolve(search.substr(1));
            }
        };
        // add iframe to DOM
        window.document.body.appendChild(iframe);
        iframe.contentWindow.addEventListener('DOMContentLoaded', iframeLoadedHandler);
        iframe.addEventListener('load', iframeLoadedHandler);
        iframe.setAttribute('src', url);
    });
};
//# sourceMappingURL=iframe.js.map